#!/usr/bin/perl
#
# BASEDIRECTORY is the directory paranix.pl and paranix.conf can be found
#
$BASEDIRECTORY = '/path/to/paranix';
#
###############################################################################
#
#    Copyright (C) 2004  BatByte Internet Software L.L.C.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################
#
###############################################################################
#
#    Version 1.0
#    PARANIX - PARANOID UNIX FILE MONITOR
#
###############################################################################
#
#
# Path to find and diff commands
#
$BINDIR = '/usr/bin';

$EMAIL;
$WARNINGS;
@FILES;
@DIRECTORIES;
$MASTERCOUNT = 0;

if($#ARGV != 0 || ($ARGV[0] ne '-i' && $ARGV[0] ne '-r')){
   print "\nAcceptable Usage:\n\n";
   print "./paranix.pl -i\n";
   print "-i = initialize or reinitialize\n\n";
   print "./paranix.pl -r\n";
   print "-r = run\n\n";
   exit(0);
}

open(CONFFILE,"$BASEDIRECTORY/para.conf") ||
   die("couldn't open para.conf with path: $BASEDIRECTORY/para.conf\n");

while($line = <CONFFILE>){
   chop($line);

   $_ = $line;
   if(/^WARNINGS/){
      ($trash,$WARNINGS) = split(/=/,$line);
      $WARNINGS =~ s/\s//g;
   }

   $_ = $line;
   if(/^EMAIL/){
      ($trash,$EMAIL) = split(/=/,$line);
      $EMAIL =~ s/\s//g;
   }

   $_ = $line;
   if(/^FILE/){
      ($trash,$file) = split(/=/,$line);
      $file =~ s/\s//g;
      push(@FILES,$file);
   }

   $_ = $line;
   if(/^DIRECTORY/){
      ($trash,$directory) = split(/=/,$line);
      $directory =~ s/\s//g;
      push(@DIRECTORIES,$directory);
   }
}

close(CONFFILE);

if($ARGV[0] eq '-i'){
   $findresults = `$BINDIR/find $BASEDIRECTORY`;
   @results = split(/\n/,$findresults);
   foreach $result (@results){
      $_ = $result;
      if(/CHECK\.\d\d/){
         unlink($result);
      }
   }

   init_files('fileinit');
   init_directories('fileinit');

   print "Initialization Of $MASTERCOUNT Files Successful\n";
}
else{
   $CHECKCOUNT = 0;

   $findresults = `$BINDIR/find $BASEDIRECTORY`;
   @results = split(/\n/,$findresults);
   foreach $result (@results){
      $_ = $result;
      if(/CHECK\.\d\d/){
         $CHECKCOUNT++;
      }
   }

   if($CHECKCOUNT == $WARNINGS){
      print "Maximum Consecutive Warnings Has Been Reached\n\n";
      print "./paranix.pl -i\n";
      print "will reinitialize the system\n";
      exit(0);
   }

   $checkfilename = 'CHECK.' . time;

   init_files($checkfilename);
   init_directories($checkfilename);

   $diff_command = "$BINDIR/diff $BASEDIRECTORY/fileinit $BASEDIRECTORY/$checkfilename";
   $diff_results = `$diff_command`;

   if(length($diff_results) > 0){
      email_warning("diff fileinit $checkfilename\n" . $diff_results);
   }
   else{
      unlink("$BASEDIRECTORY/$checkfilename");
      print "Checked $MASTERCOUNT Files, Found No Changes\n";
   } 
}

1;

sub init_files(){
   $INITNAME = shift;

   open(FILEINIT,">$BASEDIRECTORY/$INITNAME") ||
      die ("couldn't open $BASEDIRECTORY/$INITNAME");

   foreach $file (@FILES){
      $hash = generate_file_hash($file);
      print {FILEINIT} $hash . "\n";
   }

   close(FILEINIT);
}

sub init_directories(){
   $INITNAME = shift;

   open(FILEINIT,">>$BASEDIRECTORY/$INITNAME") ||
      die ("couldn't open $BASEDIRECTORY/$INITNAME");

   foreach $directory (@DIRECTORIES){
      $findresults = `$BINDIR/find $directory`;

      @results = split(/\n/,$findresults);

      foreach $result (@results){
         $hash = generate_file_hash($result);
         print {FILEINIT} $hash . "\n";
      }
   }

   close(FILEINIT);
}

sub generate_file_hash(){
   $filename = shift;

   $MASTERCOUNT++;

   ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,
    $size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($filename);

   return($filename . ",PERMISSIONS=" . $dev  . ",INODE=" . $ino  . 
          ",MODE=" . $mode  . ",HARD LINKS=" . 
          $nlink . ",UID=" . $uid  . ",GID=" . $gid  . 
          ",DEVICE=" . $rdev  . ",SIZE=" .
          $size . ",LAST MODIFIED=" .  $mtime  . 
          ",INODE CREATED=" . $ctime  . ",BLOCK SIZE=" .
          $blksize . ",BLOCKS=" . $blocks);
}

sub email_warning(){
   $message = shift;

   $output = $message . "\n\n" .
             "NOTE: If you make a change to para.conf, \n" .
             "you need to run ./paranix.pl -i to reinitialize.\n\n";

   print "Emailing Warning To: $EMAIL\n\n";
   print $output;

   $MAILPROGRAM = `which mail`;
   chop($MAILPROGRAM);

   open(MAIL, "| $MAILPROGRAM -s \"PARANIX ALERT\" $EMAIL") ||
      die("couldn't open email: $MAILPROGRAM $EMAIL\n");
   print MAIL "$output\n";
   close(MAIL);
}
